
;-------------------------------------------------------------------------------
FUNCTION IMSCALE, IMAGE, RANGE=RANGE, BOTTOM=BOTTOM, NCOLORS=NCOLORS, $
  NEGATIVE=NEGATIVE

;- Byte-scale an image (called by IMDISP)

;- Check arguments
if (n_params() ne 1) then message, 'Usage: RESULT = IMDISP_IMSCALE(IMAGE)'
if (n_elements(image) eq 0) then message, 'Argument IMAGE is undefined'

;- Check keywords
if (n_elements(range) eq 0) then begin
  min_value = min(image, max=max_value)
  range = [min_value, max_value]
endif
if (n_elements(bottom) eq 0) then bottom = 0B
if (n_elements(ncolors) eq 0) then ncolors = !d.table_size - bottom

;- Compute the scaled image
scaled = bytscl(image, min=range[0], max=range[1], top=(ncolors - 1))

;- Create a negative image if required
if keyword_set(negative) then scaled = byte(ncolors - 1) - scaled

;- Return the scaled image in the correct color range
return, scaled + byte(bottom)

END   


pro imagemap_ci_bin, image, lat, lon, maxbrit, newimage = newimage, range = range, $
  limit = limit, position = position, isotropic = isotropic, title = title, $
  xoffset = xoffset, yoffset = yoffset, xsize = xsize, ysize = ysize, $
  missing = missing, noborder = noborder, noerase = noerase, lowres = lowres, $
  current = current, b_scale=b_scale, ci_ctc


position=[0.0,0.0,1.0,1.0]
ns =1073 
nl =689   

;+
;PURPOSE:
; Display an image which has latitude and longitude defined for
; each pixel on a map projection. If a map projection is not currently
; defined, then a Mercator map projection is created which corresponds to
; the lat/lon limits of the image.
;
;CALLING SEQUENCE:
; IMAGEMAP, IMAGE, LAT, LON
;
;INPUT:
; IMAGE Array (2D) or vector (1D) of image values
; LAT Array or vector of latitudes corresponding to image values
; (degrees, -90.0 = S, +90.0 = N)
; LON Array or vector of longitudes corresponding to image values
; (degrees, -180.0 = W, +180.0 = E)
;
;OPTIONAL KEYWORDS:
; NEWIMAGE Named variable in which resampled image array is returned.
; Note that this image is always scaled to a BYTE array.
; RANGE Range of image values used for brightness scaling, [MIN,MAX]
; (default is [MIN(IMAGE),MAX(IMAGE)])
; LIMIT Limits of map display, [LATMIN,LONMIN,LATMAX,LONMAX]
; (default is [MIN(LAT),MIN(LON),MAX(LAT),MAX(LON)])
; POSITION Normalized coordinates for map display window [X1,Y1,X2,Y2]
; (default is to let MAP_SET determine the window size)
; This is useful when used in conjunction with the
; ESRG BOXPOS procedure. For example,
; IMAGEMAP, IMAGE, LAT, LON, POS = BOXPOS( /RMARG )
; will leave room at the right for a COLOR_KEY colorbar.
; ISOTROPIC If set, creates an isotropic map projection (default=non-isotropic).
; TITLE String variable containing image title (default=no title).
; XOFFSET Named variable in which the lower left device X coordinate
; of the displayed image is returned.
; YOFFSET Named variable in which the lower left device Y coordinate
; of the displayed image is returned.
; XSIZE Named variable in which the width of the displayed image
; is returned (used by devices which have scalable pixels
; such as Postscript).
; YSIZE Named variable in which the height of the displayed image
; is returned (used by devices which have scalable pixels
; such as Postscript).
; MISSING Byte value to use for missing (unfilled) portions of image
; (default is zero).
; NOBORDER If set, do not draw border around image (default=draw border).
; NOERASE If set, do not erase window before creating image (default=erase).
; LOWRES If set, draw image in low resolution mode (default=high resolution).
; CURRENT If set, use the current map projection.
;
;OUTPUT:
; The resampled image is displayed in the current graphics window
; in map coordinates. Continental outlines and lat/lon grids may be
; overlaid with MAP_CONTINENTS AND MAP_GRID.
;
;CREATED:
; Liam Gumley, CIMSS/SSEC, 26-JUL-1996
; liam.gumley@ssec.wisc.edu
; http://cimss.ssec.wisc.edu/~gumley/index.html
;
;REVISED:
; Liam Gumley, CIMSS/SSEC, 17-SEP-1996
; Modified to work with Postscript output.
; Added XOFFSET, YOFFSET, XSIZE, YSIZE keywords.
; Mercator map projection is now created only if no map projection exists.
;
; Liam Gumley, CIMSS/SSEC, 15-OCT-1996
; Added MISSING keyword to set missing values in image.
; Added NOBORDER keyword.
;
; Liam Gumley, CIMSS/SSEC, 25-NOV-1996
; Now uses MISSING keyword properly.
; Added NOERASE keyword.
; Added LOWRES keyword (useful for low resolution images, e.g. HIRS, AMSU).
;
; Liam Gumley, CIMSS/SSEC, 26-MAR-1999
; Now uses NOERASE keyword properly.
;
; Liam Gumley, CIMSS/SSEC, 13-AUG-1999
; Added CURRENT keyword.
;
;NOTES:
; (1) Hermann Mannstein (h.mannstein@dlr.de) suggested this IDL method.
; (2) This has been tested on MAS, AVHRR, GOES, and simulated MODIS data.
; It will not work well on low resolution data like HIRS or MSU.
; (3) You might run into problems with data over the poles - I've only
; tried mid-latitude imagery.
; (4) This procedure was designed for display purposes *only*.
; If you use the resampled data for any other purpose, you do so at
; your own risk.
; (5) The example takes about 15.6 seconds to execute
; on a SGI Power Indigo2 (R8000/75MHz) with 256 MB RAM.
;
;EXAMPLE:
;
;; This example is adopted from the ESRG library 'REGRID' procedure.
;; First, create latitude, longitude, and image arrays at 250x200 size.
;;
;c = complex(2,2) + cmplxgen(250,200,/center)/100
;c = c + c^2
;lon = float(c) - 100.0
;lat = 20 + imaginary(c)
;image = sqrt(abs(sin(lon*!pi)*sin(lat*!pi)))^0.3
;;
;; Resize arrays to simulate 1km resolution imagery.
;;
;lat = congrid(lat,1000,800,interp=1)
;lon = congrid(lon,1000,800,interp=1)
;image = congrid(image,1000,800,interp=1)
;;
;; Resample data to Mercator projection, and overlay coastline and grid
;;
;t0 = systime(1.0)
;imagemap,image,lat,lon
;print,'Elapsed time (sec) = ',systime(1.0)-t0
;map_continents
;map_grid
;-



;- check limit keyword

if keyword_set( limit ) then begin
;  if ((n_elements( limit ) ne 4) or (n_elements( limit ) ne 8)) then begin $
;    message, 'LIMIT must be a 4 element vector of the form [LATMIN,LONMIN,LATMAX,LONMAX]'
    
  latmin = limit( 0 )  
  lonmin = limit( 1 )
  latmax = limit( 2 )
  lonmax = limit( 3 )
endif else begin
  latmin = min( lat, max = latmax )
  lonmin = min( lon, max = lonmax )
endelse

;- check keywords

if not keyword_set( isotropic ) then isotropic = 0
if not keyword_set( noerase ) then noerase = 0

if n_elements( title ) eq 0 then title = ' '
if n_elements( missing ) eq 0 then missing = 0B
;missing = byte( ( missing > 0 ) < ( !d.table_size - 1 ) )

;;;;;;;;;;;;;;;;;;;;;;; z Buffering to run in the back ground.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  set_plot, 'z'
  device, z_buffer=0    ;,set_colors=256,set_resolution=[1000,800]

;;set_plot,'X'
;WINDOW, 4, xsize=ns, ysize=nl

!p.background = fsc_color("White",254)
;!p.color = fsc_color("Black",255)

  device = !d.name

;;  set_plot,'PS',/interpolate
;;device,/land,/helv,font_size=12,bits=24,/color,file=plot_path+'accumctc_'+date+'_'+time+'UTC.ps'

;;!p.font = 0


;;- create Mercator map projection if necessary after checking position keyword
;window,01;,xsize=xsize_window,ysize=ysize_window

;if not keyword_set( current ) then begin;;
;  if n_elements( position ) gt 0 then begin
;    if n_elements( position ) ne 4 then $
;      message, 'POSITION must be a 4 element vector of the form [X1,Y1,X2,Y2]'
;    map_set, /CYLINDRICAL, limit =limit , $
;      title = title, isotropic = isotropic, position = position, /noborder, $   
;      noerase = noerase   ;[latmin, lonmin, latmax, lonmax ]
;  endif else begin
; ;   map_set, /CONIC, 25.00, -95.00, standard_parallels=[25.0,25.0], limit =limit, $
; ;     title = title, isotropic = isotropic, /noborder, $
; ;     noerase = noerase   ;;;37.85
       map_set, /CONIC, 25.00, -95.00, standard_parallels=[25.0,25.0], limit =limit, position=position , /noborder   ;/CONIC, 37.00, -96.00, standard_parallels=[25.0,25.0], limit =limit   ;[49.9397,-130.130,50.1055,-60.8851,20.3317, -69.0821, 20.1919,-121.55]
;  endelse
;endif
;print,latmin,lonmin,latmax,lonmax
;- compute scaling range for byte image after checking range keyword

if keyword_set( range ) then begin
  if n_elements( range ) ne 2 then $
    message, 'RANGE must be a 2 element vector of the form [MIN,MAX]'
  imin = range( 0 )
  imax = range( 1 )
endif else begin
  imin = min( image, max = imax )
endelse

;- set number of samples and lines for warped image

kk=size(image, /dimensions)
;ns =1073 ;!d.x_size     ;;kk[0] ;;; 
;nl =689  ; !d.y_size  ;;;kk[1] ;;; 
if ( !d.flags and 1 ) then begin
  ns = 640L
  nl = long( float( ns ) * float( !d.y_size ) / float( !d.x_size ) )
endif

;- create resampled byte image


p = convert_coord( lon, lat, /data, /to_normal )

;IF (b_scale EQ 1) THEN BEGIN
;   newimage = replicate( 0B, ns, nl )
;   ;bad=where((image gt -4) or (image lt -60.0))   ;;;added by kaba for CTC products
;   newimage( p( 0, * ) * ( ns - 1 ), p( 1, * ) * ( nl - 1 ) ) = $
;  bytscl( image, min = imin, max = imax, top =maxbrit-1 ) + 1B   ; !d.table_size - 2
;  ;newimage(bad)=255 ;0
;ENDIF ELSE BEGIN
;  ;;BYTSCL is not giving me the expected results. please read on bytscl
;  ;;create a new bytscale where each number is just multipiled by 60 (only for CI 0,1,2,3,4"
;  ;;to create newimage with values from 0 to 240.
   newimage = replicate( 999.0, ns, nl )
  ;newimage( p( 0, * ) * ( ns - 1 ), p( 1, * ) * ( nl - 1 ) )=image 
     newimage( p( 0, * ) * ( ns ), p( 1, * ) * ( nl ) )=image
   m_pt=WHERE(newimage EQ 0)
   IF (N_ELEMENTS(m_pt) GT 1) THEN newimage(m_pt)=999.0
   
   IF (ci_ctc EQ 3) THEN BEGIN
          max_val=MAX(RANGE)
          zz=WHERE((newimage NE 999.0) AND (newimage GT max_val))
         IF (N_ELEMENTS(zz) GT 1) THEN newimage(zz)=999.0
   ENDIF
 
;ENDELSE


 ;;;keep data as is
   ;newimage = replicate( 0B, ns, nl )
   ; newimage = CONGRID(image , ns, nl, interp=0)

;- extract portion of image which fits within map boundaries


y = !y.window * nl
x = !x.window * ns

;;y=[0.00, 0.998]*nl
;;x=[0.00, 0.998]*ns

;newimage = temporary(newimage( x(0):x(1), y(0):y(1) ) )

;- compute image offset and size (device coordinates)

p = convert_coord( [ x(0), x(1) ] / float( ns ), [ y(0), y(1) ] / float( nl ), $
  /normal, /to_device )
xoffset = p(0,0)
yoffset = p(1,0)
xsize = p(0,1) - p(0,0)
ysize = p(1,1) - p(1,0)

;;- fill holes in resampled image
;
;nxr = 2
;nyr = 2
;if keyword_set( lowres ) then begin
;  nxr = 5
;  nyr = 5
;  if ( !d.flags and 1 ) then begin
;    nxr = 7
;    
;    nyr = 7
;  endif
;endif
;fill = dilate( newimage, replicate( 1, nxr, nyr ), /gray )
;loc = where( ( fill ge 1b ) and ( newimage eq 0 ), count )
;if count ge 1 then newimage( loc ) = fill( loc )
;fill = 0 ;150
;
;;- fill remaining undefined areas of image with the missing value
;
;loc = where( newimage eq 0B, count )
;if ( count ge 1 ) and ( missing gt 0B) then newimage( loc ) = missing
;
;;;- display resampled image
;;print,xoffset
;;print,yoffset
;;print,xsize
;;print,ysize
;
;;; tnewimage=ROTATE_AWIPS(newimage)
;
; pp=where(newimage(*,*) EQ 0)
; ;If(n_elements(pp) gt 1) THEN newimage(pp)=40
 
;tv, newimage, xoffset, yoffset, xsize = xsize, ysize = ysize
;- draw map borders

if not keyword_set( noborder ) then begin
  plots, [ p(0,0), p(0,1) ], [ p(1,0), p(1,0) ], /device
  plots, [ p(0,1), p(0,1) ], [ p(1,0), p(1,1) ], /device
  plots, [ p(0,1), p(0,0) ], [ p(1,1), p(1,1) ], /device
  plots, [ p(0,0), p(0,0) ], [ p(1,1), p(1,0) ], /device
endif
;
;Overlap the continental borders
	map_continents,color=254,mlinethick=1.0 ,/coasts,  /continents , limit=limit , /US  ;;/COUNTRIES,

;	;map_grid , lats=lat, lons=lon, color=0
 
;;$device,/close
   ww=SIZE(newimage, /dimensions)
   nn=ww[0] & oo=ww[1]
   tlats=FLTARR(nn,oo+1) & tlons=FLTARR(nn+1,oo)
   y_count=1 & x_count=1
   FOR k=0, ww[1] DO BEGIN
          tlats(*,k)=k+1 
   ENDFOR
  FOR l=0, ww[0] DO BEGIN
            tlons(l,*)=l+1 
  ENDFOR  
   
  qq=convert_coord(x,y, /device, /to_data)  
  z=convert_coord(tlons(*),tlats(*), /device, /to_data)

;  PRINT, 'SIZE ARRAYS:', ww
;  print, 'REMAPPED LONS RANGE:', min(z(0,*)), max(z(0,*))
;  print, 'REMAPPED LATS RANGE:', min(z(1,*)), max(z(1,*))
;  print,''
;  PRINT, 'INPUT LIMIT: ', limit
;
;  PRINT, 'Stop'
end

